/*
 * Copyright (c) 2000
 *	Bill Paul <wpaul@ee.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD$
 */

/*
 * PCCARD attachment for the Raytheon RayLink wireless network
 * adapter driver.
 *
 * Written by Bill Paul <wpaul@ee.columbia.edu>
 * Electrical Engineering Department
 * Columbia University, New York City
 */

#include "opt_inet.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/socket.h>
#include <sys/kernel.h>

#include <sys/module.h>
#include <sys/bus.h>
#include <machine/bus.h>
#include <sys/rman.h>
#include <machine/resource.h>

#include <net/if.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <net/if_ieee80211.h>
#include <net/if_media.h>

#include <dev/ray/if_rayreg.h>
#include <dev/ray/if_rayvar.h>

/*
 * Support for PCMCIA cards.
 */
static int  ray_pccard_probe		__P((device_t));
static int  ray_pccard_attach		__P((device_t));
static int  ray_pccard_detach		__P((device_t));
static int  ray_pccard_shutdown		__P((device_t));
static int  ray_alloc_mem		__P((device_t, int, int));
static int  ray_alloc_irq		__P((device_t, int, int));
static void  ray_release_resources	__P((device_t));


static device_method_t ray_pccard_methods[] = {
	/* Device interface */
	DEVMETHOD(device_probe,		ray_pccard_probe),
	DEVMETHOD(device_attach,	ray_pccard_attach),
	DEVMETHOD(device_detach,	ray_pccard_detach),
	DEVMETHOD(device_shutdown,	ray_pccard_shutdown),

	{ 0, 0 }
};

static driver_t ray_pccard_driver = {
	"ray",
	ray_pccard_methods,
	sizeof(struct ray_softc)
};

static devclass_t ray_pccard_devclass;

DRIVER_MODULE(if_ray, pccard, ray_pccard_driver, ray_pccard_devclass, 0, 0);


static int
ray_alloc_mem(dev, rid, size)
	device_t		dev;
	int			rid, size;
{
	struct ray_softc	*sc = device_get_softc(dev);
	struct resource		*res;

	res = bus_alloc_resource(dev, SYS_RES_MEMORY, &rid,
	    0ul, ~0ul, size, RF_ACTIVE);
        if (res) {
		sc->mem_rid = rid;
		sc->mem_res = res;
		return (0);
	}

	return (ENOENT);
}

static int
ray_alloc_irq(dev, rid, flags)
	device_t		dev;
	int			 rid, flags;
{
	struct ray_softc	*sc = device_get_softc(dev);
	struct resource		*res;

	res = bus_alloc_resource(dev, SYS_RES_IRQ, &rid,
	    0ul, ~0ul, 1, (RF_ACTIVE | flags));
	if (res) {
		sc->irq_rid = rid;
		sc->irq_res = res;
		return (0);
	}
	return (ENOENT);
}

static void
ray_release_resources(dev)
	device_t		dev;
{
	struct ray_softc	*sc = device_get_softc(dev);

	if (sc->mem_res) {
		bus_release_resource(dev, SYS_RES_MEMORY,
		    sc->mem_rid, sc->mem_res);
		sc->mem_res = 0;
	}
	if (sc->irq_res) {
		bus_release_resource(dev, SYS_RES_IRQ,
		    sc->irq_rid, sc->irq_res);
		sc->irq_res = 0;
	}
	return;
}

static int 
ray_pccard_detach(device_t dev)
{
	struct ray_softc	*sc = device_get_softc(dev);
	struct ifnet		*ifp = &sc->arpcom.ac_if;

	if (sc->ray_gone) {
		device_printf(dev,"already unloaded\n");
		return(0);
	}

	ray_stop(sc);
	ifp->if_flags &= ~IFF_RUNNING;
	if_detach(ifp);

	sc->ray_gone = 1;
	bus_teardown_intr(dev, sc->irq_res, sc->irq_handle);
	ray_release_resources(dev);

	return (0);
}

static int
ray_pccard_probe(device_t dev)
{
	device_set_desc(dev, "Raytheon RayLink");
	return (0);
}


static int
ray_pccard_attach(device_t dev)
{
	struct ray_softc *sc = device_get_softc(dev);
	int error;

	bzero((char *)sc, sizeof(struct ray_softc));
	sc->ray_pcdev = device_get_ivars(dev);

	ray_alloc_mem(dev, sc->mem_rid, RAY_SRAM_MEM_SIZE);
	ray_alloc_irq(dev, sc->irq_rid, 0);

	error = bus_setup_intr(dev, sc->irq_res, INTR_TYPE_NET,
	    ray_intr, sc, &sc->irq_handle);
	if (error) {
		printf("setup intr failed %d \n", error);
		ray_release_resources(dev);
		return (error);
	}
	
	sc->ray_bhandle = rman_get_bushandle(sc->mem_res);
	sc->ray_btag = rman_get_bustag(sc->mem_res);

	error = ray_attach(dev);
	return (error);
} 

static int
ray_pccard_shutdown(device_t dev)
{
	struct ray_softc	*sc;

	sc = device_get_softc(dev);
	ray_stop(sc);
	return(0);
}
