
#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <net/if.h>
#include <net/ethernet.h>

#include "if_ieee80211.h"
#include "if_rayreg.h"
#include "if_raymib.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <err.h>

static char *	ray_printhex		__P((u_int8_t *d, char *s, int len));
static void	ray_getval		__P((char *iface, struct ray_param_req *rreq));
static void	ray_getstats		__P((char *iface, struct ray_stats_req *sreq));
static void	ray_dumpstats		__P((char *iface));
static void	ray_dumpinfo		__P((char *iface));
static void	ray_setstr		__P((char *iface, u_int8_t mib, char *s));
static void	ray_setword		__P((char *iface, u_int8_t mib, u_int16_t v));
static void	ray_setval		__P((char *iface, struct ray_param_req *rreq));
static void	usage			__P((char *p));

static char	*mib_strings[] = RAY_MIB_STRINGS;
static char	*mib_help_strings[] = RAY_MIB_HELP_STRINGS;
static int	mib_sizes[] = RAY_MIB_SIZES;

static char *
ray_printhex(u_int8_t *d, char *s, int len)
{
	static char buf[3*256];
	char *p;
    	int i;

	if (len > 256)
		err(1, "Byte string too long");

	sprintf(buf, "%02x", *d);
	for (p = buf + 2, i = 1; i < len; i++) {
		sprintf(p, "%s%02x", s, *(d+i));
		p = p + 2 + strlen(s);
	}

	return(buf);
}

static void
ray_getval(char *iface, struct ray_param_req *rreq)
{
	struct ifreq ifr;
	int s;

        bzero((char *)&ifr, sizeof(ifr));

        strcpy(ifr.ifr_name, iface);
        ifr.ifr_data = (caddr_t)rreq;

        s = socket(AF_INET, SOCK_DGRAM, 0);

        if (s == -1)
                err(1, "socket");

        if (ioctl(s, SIOCGRAYPARAM, &ifr) == -1)
                err(1, "SIOCGRAYPARAM failed with failcode 0x%02x",
		    rreq->r_failcause);

        close(s);
}

static void
ray_getstats(char *iface, struct ray_stats_req *sreq)
{
	struct ifreq ifr;
	int s;

        bzero((char *)&ifr, sizeof(ifr));

        strcpy(ifr.ifr_name, iface);
        ifr.ifr_data = (caddr_t)sreq;

        s = socket(AF_INET, SOCK_DGRAM, 0);

        if (s == -1)
                err(1, "socket");

        if (ioctl(s, SIOCGRAYSTATS, &ifr) == -1)
                err(1, "SIOCGRAYSTATS failed with error %s", strerror(errno));

        close(s);
}

static void
ray_dumpinfo(char *iface)
{
	struct ray_param_req rreq;
	u_int8_t mib;

	if (iface == NULL)
		errx(1, "must specify interface name");

        bzero((char *)&rreq, sizeof(rreq));

	for (mib = RAY_MIB_NET_TYPE; mib < RAY_MIB_MAX; mib++) {
		rreq.r_paramid = mib;
		rreq.r_len = 0xFF;
		ray_getval(iface, &rreq);
		printf("%-26s\t",  mib_strings[mib]);
		switch (mib_sizes[mib]) {

		case 2:
			printf("0x%02x%02x", *rreq.r_data, *(rreq.r_data+1));
			break;

		case ETHER_ADDR_LEN:
			printf("%s",
			    ray_printhex(rreq.r_data, ":", mib_sizes[mib]));
			break;

		case IEEE80211_NWID_LEN:
			printf("%-32s", (char *)rreq.r_data);
			break;

		case 1:
		default:
			printf("0x%02x", *rreq.r_data);
			break;
		}
		printf("\t%s\n",  mib_help_strings[mib]);
	}
}


static void
ray_dumpstats(char *iface)
{
	struct ray_stats_req sreq;

	if (iface == NULL)
		errx(1, "must specify interface name");

        bzero((char *)&sreq, sizeof(sreq));

	ray_getstats(iface, &sreq);

	printf("Receiver overflows       %lu\n", (unsigned long int)sreq.rxoverflow);
	printf("Receiver checksum errors %lu\n", (unsigned long int)sreq.rxcksum);
	printf("Header checksum errors   %lu\n", (unsigned long int)sreq.rxhcksum);
	printf("Receiver signal level    %u\n", sreq.rxnoise);
}

static void
ray_setval(char *iface, struct ray_param_req *rreq)
{
	struct ifreq ifr;
	int s;

        bzero((char *)&ifr, sizeof(ifr));

        strcpy(ifr.ifr_name, iface);
        ifr.ifr_data = (caddr_t)rreq;

        s = socket(AF_INET, SOCK_DGRAM, 0);

        if (s == -1)
                err(1, "socket");

        if (ioctl(s, SIOCSRAYPARAM, &ifr) == -1) {
                err(1, "SIOCSRAYPARAM failed with failcode 0x%02x",
		    rreq->r_failcause);
	}

        close(s);
}

static void
ray_setword(char *iface, u_int8_t mib, u_int16_t v)
{
	struct ray_param_req rreq;

	if (iface == NULL)
		errx(1, "must specify interface name");

        bzero((char *)&rreq, sizeof(rreq));

	rreq.r_paramid = mib;
	rreq.r_len = mib_sizes[mib];
	switch (mib_sizes[mib]) {

	case 1:
		*rreq.r_data = (u_int8_t)(v & 0xff);
		break;

	case 2:
		*rreq.r_data = (u_int8_t)((v & 0xff00) >> 8);
		*(rreq.r_data+1) = (u_int8_t)(v & 0xff);
		break;

	default:
	    	break;
	}

	ray_setval(iface, &rreq);
}

static void
ray_setstr(char *iface, u_int8_t mib, char *s)
{
	struct ray_param_req rreq;

	if (iface == NULL)
		errx(1, "must specify interface name");
	if (s == NULL)
		errx(1, "must specify string");
	if (strlen(s) > mib_sizes[mib])
		errx(1, "string too long");

        bzero((char *)&rreq, sizeof(rreq));

	rreq.r_paramid = mib;
	rreq.r_len = mib_sizes[mib];
	bcopy(s, (char *)rreq.r_data, strlen(s));

	ray_setval(iface, &rreq);
}

static void
usage(char *p)
{
	fprintf(stderr, "usage:  %s -i iface\n", p);
	fprintf(stderr, "\t%s -i iface -o\n", p);
	fprintf(stderr, "\t%s -i iface -t tx rate\n", p);
	fprintf(stderr, "\t%s -i iface -n network name\n", p);
	fprintf(stderr, "\t%s -i iface -p port type\n", p);
fprintf(stderr, "\t%s -i iface -m mac address\n", p);
	fprintf(stderr, "\t%s -i iface -d max data length\n", p);
	fprintf(stderr, "\t%s -i iface -r RTS threshold\n", p);
	fprintf(stderr, "\t%s -i iface -f hopset\n", p);
fprintf(stderr, "\t%s -i iface -P 0|1t\n", p);
fprintf(stderr, "\t%s -i iface -S max sleep duration\n", p);
fprintf(stderr, "\t%s -i iface -Z zero out signal cache\n", p);
fprintf(stderr, "\t%s -i iface -C print signal cache\n", p);
 
	exit(1);
}

int
main(int argc, char *argv[])
{

	char *iface, *p;
	int ch, val;

	iface = NULL;
	p = argv[0];

	while ((ch = getopt(argc, argv, "hoi:c:d:f:n:p:t:W:")) != -1) {
		switch (ch) {

		case 'i':
			iface = optarg;
			break;

		case 'o':
			ray_dumpstats(iface);
			exit(0);
			break;

		case 'd':
		    	val = atoi(optarg);
			if (((val < 350) && (val != -1)) || (val > 2304))
				usage(p);
			if (val == -1)
				val = 0x7fff;
			ray_setword(iface, RAY_MIB_FRAG_THRESH, val);
			exit(0);
			break;

		case 'f':
		    	val = atoi(optarg);
			if ((val < 1) || (val >= RAY_MIB_COUNTRY_CODE_MAX))
				usage(p);
		    	ray_setword(iface, RAY_MIB_COUNTRY_CODE, val);
			exit(0);
			break;

		case 'n':
		    	ray_setstr(iface, RAY_MIB_SSID, optarg);
			exit(0);
			break;


		case 'p':
		    	val = atoi(optarg);
			if ((val < 0) || (val > 1))
				usage(p);
			ray_setword(iface, RAY_MIB_NET_TYPE, val);
			exit(0);
			break;


		case 'r':
		    	val = atoi(optarg);
			if ((val < -1) || (val > 2047))
				usage(p);
			if (val == -1)
				val = 0x7fff;
			ray_setword(iface, RAY_MIB_RTS_THRESH, val);
			exit(0);
			break;

		case 't':
		    	val = atoi(optarg);
			if ((val < 1) ||
			    (val >= RAY_MIB_BASIC_RATE_SET_MAX))
				usage(p);
		    	ray_setword(iface, RAY_MIB_BASIC_RATE_SET, val);
			exit(0);
			break;

		case 'W':
			{
				char *stringp, **ap, *av[5];
				u_int8_t mib;

				stringp = optarg;
				ap = av;
				*ap = strsep(&stringp, ":");
				ap++;
				*ap = strsep(&stringp, ":");
				mib = atoi(av[0]);
				sscanf(av[1], "%x", &val);
				printf("mib %d, val 0x%02x\n", mib, val);
				ray_setword(iface, mib, val);
			}
			exit(0);
			break;

		case 'h':
		default:
			usage(p);

		}
	}

	if (iface == NULL)
		usage(p);

	ray_dumpinfo(iface);

	exit(0);
}
